%TEST_BLGNS Test program for the BLGAUSSEXPNS function.
%   This script computes all communicabilities between some nodes of two
%   directed test networks, as well as their starting/ending convenience. The
%   results obtained by block Gauss/Antigauss quadrature are compared to those
%   obtained by expm.

%   Caterina Fenu & Giuseppe Rodriguez, University of Cagliari, Italy
%   David Martin & Lothar Reichel, Kent State University, U.S.A.
%   Email: kate.fenu@gmail.com, rodriguez@unica.it, 
%          dmarti49@kent.edu, reichel@math.kent.edu
%
%   Last revised August 4, 2013

% test matrices
testmats = {
	'celegans'
	'air500'
	};
nmatr = size(testmats,1);

clc
fprintf('\nComputation of communicabilities.\n\n')
for i = 1:nmatr
	fprintf('%3d - %s\n', i, testmats{i})
end
fprintf('\nPress enter for default values.\n')
tst = input('choose a test matrix (def. 1): ');
if isempty(tst), tst = 1; end
load(sprintf('adjacency_%s',testmats{tst}))
n = size(A,1);
fprintf('number of nodes: %d\n',n)
numnod = input('choose the number of nodes to be identified (def. 5): ');
if isempty(numnod), numnod = 5; end

matrixname = testmats{tst};

% options
clear opts
opts.gausstolq = 1e-3;		% quadrature accuracy
opts.sgcshift = 1;		% spectrum shift
opts.gaussreorth = 0;		% reorthogonalization

fprintf('Computing communicabilities by expm ')
tic
E = expm(A);
allx = E(1:numnod,1:numnod);
sc = diag(allx);
s = sum(E);
st = sum(E,2);
allx(numnod+1,:) = s(1:numnod);
allx(1:numnod,numnod+1) = st(1:numnod)';
allx(numnod+1,numnod+1) = sum(s);
fprintf('(%.2f seconds)\n',toc)

format short e

fprintf('Computing communicabilities by block Gauss-Lanczos ')
tic
W = [eye(numnod); zeros(n-numnod,numnod)];
W = [W ones(n,1)];
V = W;
[fval info_bl] = blgaussexpns(A,W,V,opts);
fprintf('(%.2f seconds)\n',toc)

fprintf('\nResults: ')
fprintf('first %d rows and columns contain communicabilities\n', numnod)
fprintf('  last row/column contain starting/ending convenience\n')
fprintf('  element (%d,%d) is meaningless.\n', numnod+1, numnod+1)
disp(fval)
fprintf('Relative errors with respect to expm: \n')
disp(abs(allx-fval)./allx)

